searchData={"content_type":"text/markdown","items":[{"doc":null,"ref":"erlarg.html","title":"erlarg","type":"module"},{"doc":"Retuns the specification for an option Name that doesn't have parameter. Example: -- help % also -h erlarg : opt ( { &quot;-h&quot; , &quot;--help&quot; } , help ) .","ref":"erlarg.html#opt/2","title":"erlarg.opt/2","type":"function"},{"doc":"Return the specification for an option Name that must be parsed with Syntax . Example: date - d &#39;now&#39; erlarg : opt ( { &quot;-d&quot; , &quot;--date&quot; } , date , string ( ) ) .","ref":"erlarg.html#opt/3","title":"erlarg.opt/3","type":"function"},{"doc":"Parses arguments into a structured data according to a given syntax","ref":"erlarg.html#parse/2","title":"erlarg.parse/2","type":"function"},{"doc":"Parses arguments into a structured data according to a given syntax and aliases","ref":"erlarg.html#parse/3","title":"erlarg.parse/3","type":"function"},{"doc":null,"ref":"erlarg.html#t:args/0","title":"erlarg.args/0","type":"type"},{"doc":null,"ref":"erlarg.html#t:opt/0","title":"erlarg.opt/0","type":"opaque"},{"doc":null,"ref":"erlarg.html#t:option_name/0","title":"erlarg.option_name/0","type":"type"},{"doc":null,"ref":"erlarg.html#t:syntax/0","title":"erlarg.syntax/0","type":"opaque"},{"doc":"# erlarg - v1.0.0\n[![erlarg CI](https://github.com/Eptwalabha/erlarg/actions/workflows/ci.yml/badge.svg)](https://github.com/Eptwalabha/erlarg/actions/workflows/ci.yml)  \n\nAn Erlang lib that parsed a list of arguments into structured data.  \nUseful for handling options/parameters of escript","ref":"readme.html","title":"erlarg - v1.0.0","type":"extras"},{"doc":"Add `erlarg` to in the `deps` of your `rebar.config`:\n```erlang\n{deps, [{erlarg, \"1.0.0\"}]}\n% or\n{depts, [{erlarg, {git, \"https://github.com/Eptwalabha/erlarg.git\", {tag, \"v1.0.0\"}}}]}\n```\nIf you're building an `escript`, add `erlarg` to the list of apps to include in the binary\n```erlang\n{escript_incl_apps, [erlarg, …]}.\n```\nfetch and compile the dependencies of your project:\n```bash\nrebar3 compile --deps_only\n```\nThat's it, you're good to go.","ref":"readme.html#installation","title":"Installation - erlarg - v1.0.0","type":"extras"},{"doc":"Imagine this command :\n```bash\n./my-script --limit=20 -m 0.25 --format \"%s%t\" -o output.tsv -\n```\nThe `main/1` function of `my-script` will receive this list of arguments:\n```erlang\n[\"--limit=20\", \"-m\", \"0.25\", \"--format\", \"%s%t\", \"-o\", \"output.tsv\", \"-\"]\n```\nThe function `erlarg:parse` will help you convert them into a structured data:\n```erlang\nmain(Args) ->\n    Syntax = {any, [erlarg:opt({\"-l\", \"--limit\"}, limit, int),\n                    erlarg:opt({\"-f\", \"--format\"}, format, binary),\n                    erlarg:opt(\"-o\", file, string),\n                    erlarg:opt(\"-\", stdin),\n                    erlarg:opt({\"-m\", \"--max\"}, max, float)\n                   ]}\n    {ok, {Result, RemainingArgs} = erlarg:parse(Args, Syntax),\n    ...\n```\nFor this example, `parse` will return this proplist:\n```erlang\n % Result\n[{limit, 20},\n {max, 0.25},\n {format, <<\"%s%t\">>},\n {file, \"output.tsv\"},\n stdin].\n```\nThe functions `erlarg:parse/2` & `erlarg:parse/3` will transform a list of arguments into a structured data.\n- `Args`: A list of arguments (generaly what's given to `main/1`)\n- `Syntax`: The syntax (or the specification) that describes how the arguments should be parsed\n- `Aliases`: [optional] the list of options, types or sub-syntax that are referenced in `Syntax`","ref":"readme.html#how-does-it-work","title":"How does it work ? - erlarg - v1.0.0","type":"extras"},{"doc":"The syntax will describe to the parser how to handle each arguments (`Args`).\nIt will consume each argument one by one while building the structured data.\n\nA syntax could be any of those things:\n- a type\n- a named type\n- a custom type\n- an option\n- an alias\n- a sub-syntax (which is a syntax itself)\n- a syntax operator\n- a list of all the above\n\nIt can be pretty complex, but for now, let's go simple.\n\nImagine this fictionnal script `print_n_time` that takes a string and an integer as argument\n```bash\n# this will print the string \"hello\" 3 times\n$ print_n_time hello 3\n```\nHere's the simplest spec needed to handle the arguments:\n```erlang\nSyntax = [string, int].\nerlarg:parse(Args, Syntax). % here Args = [\"hello\", \"3\"]\n{ok, {[\"hello\", 3], []}} % erlang:parse/2 result\n```\nWe explicitly asked the parser to handle two arguments, the first  must  be a `string`, the second  must  be an `int`.  \nIf if the parsing is successful, it will return the following tuple:\n```erlang\n{ok, {Data, RemainingArgs}}.\n```\nWhere `Data` is the structured data generated by the parser (`[\"hello\", 3]`) and `RemainingArgs` is the list of arguments not consumed by the parser (`[]`).","ref":"readme.html#syntax","title":"Syntax - erlarg - v1.0.0","type":"extras"},{"doc":"If the parser encounter a problem with an argument, it will fail and return the nature of the problem:\n```erlang\n> erlarg:parse([\"world\"], [int]).\n{error, {not_int, \"word\"}} % it failed to convert the word \"world\" into an int\n```\nor\n```erlang\n> erlang:parse([\"one\"], [string, string]). % expect two strings but only got one\n{error, {missing, arg}}\n```\n> [!TIP]\n> These errors can be used to explain to the user what's wrong with the command it typed","ref":"readme.html#parsing-failure","title":"Parsing failure - erlarg - v1.0.0","type":"extras"},{"doc":"Remaining args are the arguments not consumed by the parser when this one terminates successfuly.  \nIf we add some extra arguments at the end of our command:\n```bash\n$ print_n_time hello 3 some extra arguments\n```\nthis time, calling `erlarg:parse/2` with the same syntax as before will give this result:\n```erlang\nSyntax = [string, int].\n{ok, {_, RemainingArgs}} = erlarg:parse(Args, Syntax).\n[\"some\", \"extra\", \"arguments\"] % RemainingArgs\n```\nThe parser will consume the two first arguments, the remaining argument will be returned in the `RemainingArgs`.\n\n> [!NOTE]\n> Having unconsumed arguments does not generate an error","ref":"readme.html#remaining-args","title":"Remaining Args - erlarg - v1.0.0","type":"extras"},{"doc":"The parser can convert the argument to more types than just `string` and `int`.  \nHere are all the types currently available :\n\n- `int`: cast the argument into an int\n- `float`: cast the argument into a float (will cast int into float)\n- `number`: cast the argument into an int. If it fails it will cast the argument into a float\n- `string`: returns the given argument\n- `binary`: cast the argument into a binary list\n- `atom`: cast the arg to an atom\n- `bool`: return the boolean value of the arg\n\n| syntax | arg | result | note |\n|---|---|---|---|\n| int | \"1\" | 1 |-|\n| int | \"1.2\" | error | not an int |\n| float | \"1.2\" | 1.2 |-|\n| float | \"1\" | 1.0 | cast int into float |\n| float | \"1.234e2\" | 123.4 |-|\n| number | \"1\" | 1 |-|\n| number | \"1.2\" | 1.2 |-|\n| string | \"abc\" | \"abc\" |-|\n| binary | \"äbc\" | <<\"äbc\"/utf8>> | use `unicode:characters_to_binary`|\n| atom | \"super-top\" | 'super-top' |-|\n\nthe `bool` conversion:\n\n| arg | bool | note |\n|---|---|---|\n| \"true\" | true | case insensitive |\n| \"yes\" | true | |\n| \"abcd\" | true | any non-empty string |\n| \"1\" | true | |\n| \"0.00001\" | true | |\n| \"false\" | false | case insensitive |\n| \"no\" | false | |\n| \"\" | false | empty-string|\n| \"0\" | false | |\n| \"0.0\" | false | |\n\n> [!TIP]\n> converting an argument into `string`, `binary`, `bool` or `atom` it will always succeed.\n\nIf you need more complicated \"type\", see the chapter on [Custom types](#custom-types)","ref":"readme.html#types","title":"Types - erlarg - v1.0.0","type":"extras"},{"doc":"Converting an argument into a specific type is important, but it doesn't really help us understand what these values are for:  \n```erlang\n> Syntax = [string, int].\n> {ok, {Result, _}} = erlarg:parse([\"hello\", \"3\"], Syntax).\n[\"hello\", 3]. % Result\n```\nTo avoid this issue, you can give \"name\" to the parsed parameters with the following syntax:\n```erlang\n{Name :: atom(), Type :: base_type()}\n```\nIf we rewrite the syntax as such:\n```erlang\nSyntax = [{text, string()}, {nbr, int}].\n{ok, {Result, _}} = erlarg:parse([\"hello\", \"3\"], Syntax).\n[{text, \"hello\"}, {nbr, 3}] % Result\n```\nyou can even name a list of parameters if you want:\n```erlang\nSyntax = [{a, [string, {a2, float}]}, {b, binary}],\n{ok, {Result, _}} = erlang:parse([\"abc\", \"2.3\", \"bin\"], Syntax).\n[{a, [\"abc\", {a2, 2.3}]}, {b, <<\"bin\">>}] % Result\n```","ref":"readme.html#naming-parameters","title":"Naming parameters - erlarg - v1.0.0","type":"extras"},{"doc":"Naming and casting parameters into types is neat, but most programs use options. An option is an argument that usually (not always…) starts with dash and has zero or more parameters.  \n```bash\n$ date -d --utc --date=STRING\n```\nOption can have several formats a short one (a dash followed by a letter eg. `-v`) and/or a long one (double dash and a word eg. `--version`)\n\nThis table summarizes the formats handled/recognized by the parser:\n\n| format | note |\n|---|---|\n| -s | |\n| -s  VALUE  | |\n| -s VALUE  | same as `-s VALUE` |\n| -abc  VALUE  | same as `-a -b -c VALUE` |\n| -abc VALUE  | same as `-a -b -c VALUE` |\n| --long | |\n| --long  VALUE  | |\n| --long= VALUE  | |\n\nIn this chapter, we'll see how to tell the parser how to recognise three kind of options:\n- option without parameter\n- option with parameters\n- option with sub options","ref":"readme.html#options","title":"Options - erlarg - v1.0.0","type":"extras"},{"doc":"```bash\n$ grep -v \"bad\"\n$ grep --invert-match \"bad\"\n```\nWe can define this option with `erlarg:opt` like so:\n```erlang\n> Syntax = [erlarg:opt({\"-v\", \"--invert-match\"}, invert_match)].\n> {ok, {Result, _}} = erlarg:parse([\"-v\"], Syntax),\n[invert_match] % Result\n```\nThe first parameter of `erlarg:opt` is the option:\n```erlang\n{\"-s\", \"--long\"} % short and long options\n\"-s\" % only short option\n{\"-s\", undefined} % same as above\n{undefined, \"--long\"} % only long option\n```\n\nThe second parameter is the name of the option, in this case `invert_match`","ref":"readme.html#option-without-parameter","title":"option without parameter - erlarg - v1.0.0","type":"extras"},{"doc":"Option can have parameters\n```bash\n$ date --date 'now -3 days'\n$ date --date='now -3 days'\n$ date -d'now -3 days'\n```\n```erlang\n> Syntax = [erlarg:opt({\"-d\", \"--date\"}, date, string)].\n> {ok, {Result, _}} = erlarg:parse([\"--date\", \"now -3 days\"], date, string).\n[{date, \"now -3 days\"}] % Result\n```\nThe third parameter is the syntax of the parameters expected by the option. In this case after matching the argument `--date` this option is expecting a string (`\"now -3 days\"`).  \n\nMaybe one of the option of your program is expecting two parameters ?\nNo problem :\n```erlang\nerlang:opt({\"-d\", \"--dimension\"}, dimension, [int, string]}).\n[{dimension, [3, \"inch\"]}] % Result for \"-d 3 inch\"\n```\nYou can even use name\n```erlang\nerlang:opt({\"-d\", \"--dimension\"}, dimension, [{size, int}, {unit, string}]).\n[{dimension, [{size, 3}, {unit, \"inch\"}]}] % Result for \"-d 3 inch\"\n```","ref":"readme.html#option-with-parameter-s","title":"option with parameter(s) - erlarg - v1.0.0","type":"extras"},{"doc":"Because the third parameter is a syntax, and because an option is a syntax itself, that means you can put options into option :\n```bash\n$ my-script --opt1 -a \"param of a\" -b \"param of opt1\" --opt2 …\n```\nIn this fictionnal program, the option `--opt1` has two sub-options (`-a` that expects a parameter and `-b` that doesn't). We can define `opt1` this way:\n```erlang\nOpt1 = erlarg:opt({\"-o\", \"--opt1\"}, % option\n                  opt1, % option's name \n                  [erlarg:opt(\"-a\", a, string), % sub-option 1\n                   erlarg:opt(\"-b\", b),  % sub-option 2\n                   {value, string} % the param under the name 'value'\n                  ]).\n{ok, {Result, _}} = erlarg:parse([\"--opt1\", \"-a\", \"abc\", \"-b\", \"def\"], Opt1).\n[{opt1, [{a, \"abc\"}, b, {value, \"def\"}]}] % Result\n```\nWell… that's quite unreadable… fortunately, you can use `Aliases` to avoid this mess.","ref":"readme.html#option-with-sub-option-s","title":"option with sub-option(s): - erlarg - v1.0.0","type":"extras"},{"doc":"Aliases, let you define all your options, sub-syntax and custom types in a map. It helps keep the Syntax clear and readable.\n```erlang\nAliases = #{\n    option1 => erlarg:opt({\"-o\", \"--opt1\"}, opt1, [opt_a, opt_b, {value, string}]),\n    option2 => erlarg:opt({undefined, \"--opt2\"}, opt2),\n    opt_a => erlarg:opt(\"-a\", a, string),\n    opt_b => erlarg:opt(\"-b\", b)\n},\nSyntax = [option1, option2],\n{ok, {Result, _}} = erlarg:parse([\"--opt1\", \"-a\", \"abc\", \"-b\", \"def\", \"--opt2\"],\n                                 Syntax, Aliases).\n[{opt1, [{a, \"abc\"}, b, {value, \"def\"}]}, opt2] % Result\n```\nHere `Syntax` is a list of two aliases, `option1` and `option2`","ref":"readme.html#aliases","title":"Aliases - erlarg - v1.0.0","type":"extras"},{"doc":"Operator tells the parser how to handle a list of syntax\n\n### `sequence` operator\nTake the following syntax:\n```erlang\n[opt({\"-d\", \"--date\"}, date, string), opt({\"-u\", \"--utc\"}, utc)]\n```\nIt would parse this command without problem:\n```bash\n$ date -d \"now -3 days\" --utc # yay!\n```\nBut will crash with this one:\n```bash\n$ date --utc --date=\"now -3 days\" # boom !\n```\nWhy ? Aren't these two commands identical ?  \nThat's because a list of syntax is considered by the parser as a `sequence` operator :\n```\n[syntax1, syntax2, …]\n```\nA `sequence` is expecting the arguments to match in the same order as the elements of the list. The first argument must match `syntax1`, the second `syntax2`, …) if any fails, the whole sequence fails.  \n\nAll elements of the list must succeed in order for the operator to succeed.  \n\n| syntax | args | result | note |\n|---|---|---|---|\n| [int, string] | [\"1\", \"a\"] | [1, \"a\"] | |\n| [int] | [\"1\", \"a\"]  | [1] | remaining: [\"a\"] |\n| [int, int] | [\"1\", \"a\"] | error | \"a\" isn't an int |\n| [int, string, int] | [\"1\", \"a\"] | error | missing a third argument |\n\nSo how to parse arguments if we're not sure of they order… moreover, some option are… optionnal ! how do we do ?\nThat's where the `any` operator comes to play.  \n\n### `any` operator\nformat:\n```\n{any, [syntax1, syntax2, …]}\n```\nThe parser will try to consume arguments as long as one of syntax matches. If an element of the syntax fails, the operator fails.  \n\n| syntax | args | result | note |\n|---|---|---|---|\n| {any, [int]} | [\"1\", \"2\", \"abc\"] | [1, 2] | remaining: [\"abc\"] |\n| {any, [{key, int}]} | [\"1\", \"2\"] | [{key, 1}, {key, 2}] | |\n| {any, [int, {s, string}]} | [\"1\", \"2\", \"abc\", \"3\"] | [1, 2, {s, \"abc\"}, 3] | |\n| {any, [string]} | [\"1\", \"-o\", \"abc\", \"3\"] | [\"1\", \"-o\", \"abc\", \"3\"] | even if \"-o\" is an option |\n\nNo matter the number of matching element, `any` will always succeed. If nothing matches no arguments will be consumed.\n\n> [!NOTE]\n> Keep in mind that if the list given to `any` contains types like `string` or `binary`, it will consume all the remaining arguments.  \n> `{any, [string, custom_type]}`, `custom_type` will never be executed because the type `string` will always consume argument\n\n### `first`\nformat:\n```\n{first, [syntax1, syntax2, …]}\n```\nThe parser will return the first element of the syntax to succeed.\nIt'll fail if no element matches.  \nThe following table use `Args = [\"a\", \"b\", \"1\"]`\n\n| syntax | result | remaining |\n|---|---|---|\n| {first, [int]} | [1] | [\"2\", \"a\", \"3\", \"b\"] |\n| {first, [{opt, int}]} | [{opt, 1}] | [\"a\", \"3\", \"b\"] |\n| {any, [int, {b, binary}]} | [1, 2, {b, <<\"a\">>}, 3, {b, <<\"b\">>}] | [] |\n| {any, [string]} | [\"1\", \"2\", \"a\", \"3\", \"b\"] | [] |","ref":"readme.html#syntax-operators","title":"Syntax operators - erlarg - v1.0.0","type":"extras"},{"doc":"Sometime, you need to perfom some operations on an argument or do more complexe verifications. This is what custom type is for.  \nA custom type is a function that takes a list of arguments and return the formated / checked value to the parser:\n```erlang\n-spec fun(Args) -> {ok, Value, RemainingArgs} | Failure) where\n    Args :: args(),\n    Value :: any(),\n    RemainingArgs :: args(),\n    Failure :: any().\n```\n- `Args`: The list of arguments not yet consumed by the parser\n- `Value`: The Value you want to return to the parser\n- `RemainingArgs`: The list of arguments your function didn't consumed\n- `Failure`: some explanation on why the function didn't accept the argument\n\n**Example 1**:  \nLet say your script has an option `-f FILE` where `FILE` must be an existing file. In this case the type `string` won't be enought. You could write your own function to perform this check:\n```erlang\nexisting_file([File | RemainingArgs]) ->\n    case filelib:is_regular(File) of\n        true -> {ok, File, RemainingArgs};\n        _ -> {not_a_file, File}\n    end.\n```\nTo use your custom type:\n```erlang\nSpec = #{\n    syntax => {any, [file]},\n    definitions => #{\n        file => erlarg:opt({\"-f\", \"--file\"}, existing_file),\n        existing_file => fun existing_file/1\n    }\n}.\n```\nor directly as a syntax:\n```erlang\nSpec = {any, [{file, erlarg:opt({\"-f\", \"--file\"}, fun existing_file/1)}]}.\n```\n\n**Example 2**:  \nIn this case, your script needs to fetch the informations of a particular user from a config file with the option `--consult USERS_FILE USER_ID` where `USERS_FILE` is the file containing the users data and `USER_ID` is the id of the user:\n```erlang\nget_user_config([DatabaseFile, UserID | RemainingArgs]) ->\n    case file:consult(DatabaseFile) of\n        {ok, Users} ->\n            case proplists:get_value(UserID, Users, not_found) of\n                not_found -> {user_not_found, UserID};\n                UserData -> {ok, UserData, RemainingArgs}\n            end;\n        Error -> {cannot_consult, DatabaseFile, Error}\n    end;\nget_user_config(_) ->\n    {badarg, missing_arguments}.\n```","ref":"readme.html#custom-types","title":"Custom types - erlarg - v1.0.0","type":"extras"},{"doc":"# Apache License\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html","title":"Apache License","type":"extras"},{"doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions","title":"1. Definitions. - Apache License","type":"extras"},{"doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license","title":"2. Grant of Copyright License. - Apache License","type":"extras"},{"doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license","title":"3. Grant of Patent License. - Apache License","type":"extras"},{"doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution","title":"4. Redistribution. - Apache License","type":"extras"},{"doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions","title":"5. Submission of Contributions. - Apache License","type":"extras"},{"doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks","title":"6. Trademarks. - Apache License","type":"extras"},{"doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty","title":"7. Disclaimer of Warranty. - Apache License","type":"extras"},{"doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability","title":"8. Limitation of Liability. - Apache License","type":"extras"},{"doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2025, eptwalabha  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability","title":"9. Accepting Warranty or Additional Liability. - Apache License","type":"extras"}],"producer":{"name":"ex_doc","version":"0.35.1"},"proglang":"erlang"}